{

=pod

=head1 NAME

WWW::NYTimes::PSWriter

=head1 SUMMARY

TBW

=head1 DESCRIPTION 

TBW

=cut

package WWW::NYTimes::PSWriter;
use strict;

use Carp;
use FileHandle;

use XML::Parser::PerlSAX;
use PostScript::TextBlock;
use PostScript::Elements;

my $first_h = 20;
my $first_y = 742;

my $left    = 50;
my $right   = 562;
my $bottom  = 75;

my $h = $first_h;
my $y = $first_y;

my $is_book    = 0;
my $is_part    = 0;
my $is_article = 0;

my $booktitle    = undef;
my $parttitle    = undef;
my $articletitle = undef;

my $lasttag = undef;
my $okay    = 0;

my $text     = "";
my $numpages = 1;
my %info     = ();

my $fh = undef;

my $curdate  = undef;
my $curstory = undef;
my $curpage  = undef;

my $story_num = 0;

my %toc       = ();
my @stories   = ();

=pod

=head1 PUBLIC METHODS

=cut

=pod

=head2 &parse($string,$outfile)

=cut

sub parse { return &_create(@_); }

=pod

=head2 &parse_from_file($file,$outfile)

=cut

sub parse_from_file { return &_create(@_); }

=pod 

=head1 PRIVATE METHODS

==cut

=pod

=head2 &_create(%args)

Valid arguments are 

=over 4

=item B<file>

=item B<string>

=item B<outfile>

=back

=cut

sub _create {
    my $args = { @_ };

    my $ps = $args->{'outfile'};

    $fh = FileHandle->new();
    $fh->open(">$ps");

    &PS::start();
    &PS::new_page();

    if (my $title = $args->{'title'}) {
	&PS::print_box("$title",size=>24,height=>40,font=>"Helvetica");
	&PS::print_line();
	&PS::print_break();
    }

    print "Printing index...";
    &print_index(%$args);
    print "ok\n";

    &PS::end_page();
    &PS::new_page();

    print "Printing articles...";
    &print_articles(%$args);
    print "ok\n";

    &PS::finish();

    print "Setting page numbers...";
    &set_pagenums($ps);
    print "ok\n";

    return 1;
}

sub set_pagenums {
    my $outfile = shift;
    my $tmpfile = "$outfile.tmp";

    my $read  = FileHandle->new();
    my $write = FileHandle->new();

    $read->open("<$outfile");
    $write->open(">$tmpfile");

    my $i = 0;
    while (<$read>) {
	my $line = $_;
	chomp $line;

	if ($line =~ /(.*)(\.\.\.)(\{\{)(\d+)(\}\})(.*)/) {
	    print $write "$1$2$stories[$4]$6\n";
	} else {
	    print $write $line,"\n";
	}
	$i++;
    }

    $read->close();
    $write->close();

    rename $outfile, "$outfile.old";
    rename $tmpfile, $outfile;

    unlink "$outfile.old";
    return 1;
}

sub print_index {
    my $handler = Index->new();
    &do_parse($handler,&parse_args(@_));
    return 1;
}

sub print_articles {
    my $handler = Article->new();
    &do_parse($handler,&parse_args(@_));
    return 1;
}

sub parse_args {
    my $args = { @_ };
    my %parseargs = ();

    if ($args->{'string'}) {
	$parseargs{ String }   = $args->{'string'};
    } else {
	$parseargs{ SystemId } = $args->{'file'};
    }

    return \%parseargs;
}
sub do_parse {
    my $handler = shift;
    my $args    = shift;
    my $parser  = XML::Parser::PerlSAX->new(Handler=>$handler);
    my $result  = $parser->parse( Source=>$args);
    return $result;
}

package PS;

sub start {
    print $fh
	"%!PS-Adobe-3.0\n",
	"%%Creator: The Man.\n",
	"%%Pages: (atend)\n",
	"%%BoundingBox: 0 0 612 792\n",
	"%%EndComments\n",
	"%%BeginProlog\n",
	"%%EndProlog\n";
    return 1;
}

sub finish {
    print $fh
	"%%Trailer\n",
	"%%Pages: $numpages\n",
	"%%EOF\n";
    return 1;
}

sub print_line {
    my $len = shift || 562;
    my $el = PostScript::Elements->new();
    $el->addLine(points=>[50,$y,$len,$y]);
    print $fh $el->Write();
    return 1;
}

sub print_break {
    return if (keys %info);

    my $box = PostScript::TextBlock->new();
    $box->addText(text=>"",size=>4);

    $h = 15;
    
    my $x = $y - $h;

    if ($x < $bottom) { 
	if (&end_page()) {
	    &new_page();
	}
    }
    
    my ($code,$remainder) = $box->Write(512,$h,50,$y);
    print $fh $code;
    
    $y = $y - $h;
    $h = $first_h;

    return 1;
}

sub print_box {
    my $text = shift;
    my $args = { @_ };

    unless ($text =~ /\W/) {
	return;
    }
    
    $text =~ s/^(\s)*//;
    
    my $size  = $args->{'size'}   || 8;
    my $new_h = $args->{'height'} || undef;

    my $this_left = $args->{'left'} || $left;
    my $width = $right - $this_left;

    if ($new_h) { $h = $new_h; }

    my $tries     = 0;
    my $lastchars = 0;

    while ($text) {
	my $box = PostScript::TextBlock->new();
	$box->addText(text=>"$text",size=>$size,font=>$args->{'font'});

	my $x = $y - $h;

	if ($x < $bottom) {	    
	    if (&end_page()) {
		&new_page();
	    }
	}

	my ($code,$remainder) = $box->Write($width,$h,$this_left,$y);

	print $fh $code;

	my $chars = length($remainder->[0]->{'text'});
	$text     = substr($text,(length($text)-$chars),length($text));
	# print "chars left : $chars\n";	

	if ($tries == 3) {
	    Carp::carp "Unable to set text after (3) tries.\nPage $numpages, $curstory\n";
	    $text = undef;
	}

	$tries++ if ($chars eq $lastchars);
	$lastchars = $chars;

	$y = $y - $h;
    }

    if ($new_h) { $h = $first_h; }

    return 1;
}

sub end_page {

    # Nothing has been printed, so why are
    # ending a perfectly good page.
    return 0 if ($y eq $first_y);

    if ($y > $bottom) {
	my $f = ($first_y - $bottom) - $y;
	my $box = PostScript::TextBlock->new();
	$box->addText(text=>"",size=>10);

	my ($code) = $box->Write(512,$f,50,$y);
	print $fh $code;
    }

    $y = $bottom;
    &print_line();

    my $footer = "$numpages | $curdate ";

    if ($curstory) {
	$footer .= "| $curstory ( $curpage )";
    }

    my $box    = PostScript::TextBlock->new();
    $box->addText(text=>$footer,size=>8,font=>"Helvetica");

    my ($foot,$remainder) = $box->Write(512,$h,50,$y);
    
    if (my $wtf = $remainder->[0]->{'text'}) { 
	#print "Failed to add footer : $wtf\n";
    }
    
    print $fh 
	$foot,
	"showpage\n";

    $curpage++;
    $numpages++;

    $h = $first_h;
    $y = $first_y;

    return 1;
}

sub new_page {
    print $fh
	"%%Page: $numpages\n";
    
    return 1;
}

package Index;

sub new {
    my $pkg = shift;
    return bless {},$pkg;
}

sub start_element {
    my $self = shift;
    my $el   = shift;

    my $tag = $el->{'Name'};

    if ($tag eq "part") {
	$info{"_okay"} = 1;
    }

    if ($tag eq "book") {
	$is_book = 1;
    }

    $lasttag = $tag;
}

sub end_element {
    my $self = shift;
    my $el   = shift;

    my $tag = $el->{'Name'};

    if ($tag eq "articleinfo") {

	my $title  = "$info{'title'}....{{$story_num}}";
	my $author = "$info{'firstname'} $info{'surname'}";

	my @names = split(/\s/,$author);
	map { $_ = lc $_; $_ = ucfirst $_; } @names;
	$author = join(" ", @names);

	my $blurb  = "$info{'para'}";
	$blurb    .= " -- $author" if ($author);

	&PS::print_box($title,left=>100,font=>"Helvetica");	
	&PS::print_box($blurb,left=>125,size=>8,font=>"Times-Italic",height=>16);

	%info = ();
	$story_num++;
    }

    if (($tag eq "title") && ($is_book)){
	#print $booktitle,"\n";
	&PS::print_box(" $booktitle",size=>12,font=>"Helvetica");
	$is_book   = undef;
	$booktitle = undef;
    }

    $lasttag = $tag;
}

sub characters {
    my $self  = shift;
    my $chars = shift;

    my $data = $chars->{'Data'};

    if (($is_book) && ($lasttag eq "title")) {
	$booktitle .= $data;
    }

    if (($info{"_okay"}) && ($data)) {
	$info{$lasttag} = $data;
    }
}

package Article;

my $ulink = undef;

sub new {
    my $pkg = shift;
    return bless {},$pkg;
}

sub start_element {
    my $self  = shift;
    my $el    = shift;

    my $tag   = $el->{'Name'};

    if ($tag eq "part") {
	$info{"_okay"} = 1;
    }

    if ($tag eq "book") {
	$is_book = 1;
    }

    if ($tag eq "ulink") {
	$ulink = $el->{'Attributes'}->{'url'};
    }

    $lasttag = $tag;
}

sub end_element {
    my $self = shift;
    my $el = shift;

    my $tag = $el->{'Name'};

    if ($tag eq "articleinfo") {

	# The NYT-online often publishes the same story
	# in multiple sections. If we have already printed
	# a particular story, we don't need to print it
	# again. However, we do want to register the page 
	# num of the first instance as the page num for all
	# subseuquent ones. This way, the index will 
	# still list all the instances, but only point to 
	# one of them.

	if (my $npages = $toc{ $info{'title'} }) {
	    push  @stories, $npages;
	    return;
	}

	&PS::print_box($info{'title'},size=>12,height=>30,font=>"Helvetica");
	&PS::print_line();

	if ($info{'surname'} || $info{'firstname'}) {
	    &PS::print_box("$info{'firstname'} $info{'surname'}");
	    &PS::print_line();
	}

	$curstory = $info{'title'};
	$curpage = 1;

	$toc{ $info{'title'} } = $numpages;
	push @stories,$numpages;

	%info = ();
	$okay = 1; 

	&PS::print_break();
    }
	
    if ($tag eq "para") {
	&PS::print_box($text);
	&PS::print_break();
	$text = "";
    }

    if ($tag eq "article") { 

	&PS::print_box($ulink,size=>8,font=>"Times-Italic",);
	$ulink = "";

	if (&PS::end_page()) {
	    &PS::new_page();
	} 

	$okay = 0; 

	$curstory = undef;
	$curpage  = undef;
    }

    if ($tag eq "book") {
	$booktitle = undef;
    }

}

sub characters {
    my $self  = shift;
    my $chars = shift;

    my $data = $chars->{'Data'};

    if (($is_book) && ($lasttag eq "title")) {
	$booktitle .= $data;
    }

    if (($info{"_okay"}) && ($data)) {
	$info{$lasttag} = $data;
    }

    if (($okay) && ($data)) {
	$text .= $data;
    }
}

=pod

=head1 VERSION 

1.0

=head1 AUTHOR 

Aaron Straup Cope

=head1 LICENSE

Copyright (c) 2001-2002 Aaron Straup Cope. All Right Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
